#ifndef _MSTBRIDGE_H /* -*- mode: C; -*- */
#define _MSTBRIDGE_H

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:08 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */

/** @file
 * MathEngine Collision <-> Dynamics Toolkit Bridge API.
 *
 * This is used by MstUniverseStep to set the physical parameters of
 * dynamics contacts (such as friction, restitution etc.) based on the
 * materials of the two geometries in contats. It also holds an optional
 * callback for each MstMaterialID pair.
 */


#include <MstTypes.h>

/** @def MstBridgeGetDefaultMaterial() */
/** MstBridgeGetDefaultMaterial */

#define MstBridgeGetDefaultMaterial() (0)


#ifdef __cplusplus
extern "C" {
#endif

/** MstBridgeContactBufferSize */
extern const int MstBridgeContactBufferSize;
/** MstBridgeResultBufferSize */
extern const int MstBridgeResultBufferSize;

/** MstBridgeCreate */
MEPUBLIC
MstBridgeID           MEAPI MstBridgeCreate(McdFrameworkID frame,
                                const unsigned int maxMaterials);

/** MstBridgeDestroy */
MEPUBLIC
void                  MEAPI MstBridgeDestroy(const MstBridgeID b);

/** MstBridgeStepAll */
MEPUBLIC
void                  MEAPI MstBridgeStepAll(const MstBridgeID bridge,
                                const MeReal stepSize);
/** MstBridgeUpdateContacts */
MEPUBLIC
void                  MEAPI MstBridgeUpdateContacts(const MstBridgeID b,
                                const McdSpaceID s,const MdtWorldID w);
/** MstBridgeUpdateTransitions */
MEPUBLIC
void                  MEAPI MstBridgeUpdateTransitions(const MstBridgeID b,
                                const McdSpaceID s,const MdtWorldID w);

/** MstBridgeGetContactParams */
MEPUBLIC
MdtContactParamsID    MEAPI MstBridgeGetContactParams(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2);
/** MstBridgeGetPerPairCB */
MEPUBLIC
MstPerPairCBPtr       MEAPI MstBridgeGetPerPairCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2);
/** MstBridgeGetPerContactCB */
MEPUBLIC
MstPerContactCBPtr    MEAPI MstBridgeGetPerContactCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2);
/** MstBridgeGetNewMaterial */
MEPUBLIC
MstMaterialID         MEAPI MstBridgeGetNewMaterial(const MstBridgeID b);

/** MstBridgeGetIntersectCB */
MEPUBLIC
MstIntersectCBPtr     MEAPI MstBridgeGetIntersectCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2);

/** MstBridgeSetPerPairCB */
MEPUBLIC
void                  MEAPI MstBridgeSetPerPairCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2,
                                const MstPerPairCBPtr cb);
/** MstBridgeSetPerContactCB */
MEPUBLIC
void                  MEAPI MstBridgeSetPerContactCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2,
                                const MstPerContactCBPtr cb);
/** MstBridgeSetIntersectCB */
MEPUBLIC
void                  MEAPI MstBridgeSetIntersectCB(const MstBridgeID b,
                                const MstMaterialID m1,const MstMaterialID m2,
                                const MstIntersectCBPtr cb);
/** MstBridgeSetModelPairBufferSize */
MEPUBLIC
void                  MEAPI MstBridgeSetModelPairBufferSize(
                                const MstBridgeID b,const unsigned int s);
/** MstBridgeSetContactBufferSize */
MEPUBLIC
void                  MEAPI MstBridgeSetContactBufferSize(const MstBridgeID b,
                                const unsigned int s);
#ifdef __cplusplus
}
#endif

#endif
